extends BaseMenuScreen

enum SelectedMenu {resume, new_game, options, credits, quit}

var _selected_menu := SelectedMenu.resume

func _init():
	if Game.state.language_code == "":
		var os_locale := OS.get_locale_language()
		if os_locale == "fr":
			TranslationServer.set_locale(os_locale)
			Game.state.language_code = "fr"
		else:
			Game.state.language_code = "en"
		Game.persist_state()
	else:
		TranslationServer.set_locale(Game.state.language_code)

func _ready():
	if Game.state.level == 0:
		_selected_menu = SelectedMenu.new_game
		$ResumeLabel.visible = false
		$MenuSelector.global_position = $NewGameLabel.get_global_rect().get_center()
	
	_update_touch_interface()
	Input.joy_connection_changed.connect(_joy_connection_changed)
	
	if DisplayServer.window_get_mode() == DisplayServer.WINDOW_MODE_EXCLUSIVE_FULLSCREEN: 
		Input.mouse_mode = Input.MOUSE_MODE_HIDDEN
	else:
		Input.mouse_mode = Input.MOUSE_MODE_VISIBLE
	
	Musics.play_main_menu_music()

func _joy_connection_changed(_device: int, _connected: bool):
	_update_touch_interface()

func _update_touch_interface():
	$MenuSelector.visible = not InputUtils.is_in_touch_mode()
	$PrivacyPolicyLabel.visible = OS.get_name() == "Android"
	$QuitLabel.visible = OS.get_name() != "iOS" # does not work on iOS

func _unhandled_input(_event: InputEvent):
	super(_event) # needed to detect menu actions
	
	if _is_menu_up_pressed:
		match _selected_menu:
			SelectedMenu.resume:
				if OS.get_name() == "iOS": # quit is hidden on iOS
					_set_selected_menu(SelectedMenu.credits)
				else:
					_set_selected_menu(SelectedMenu.quit)
			SelectedMenu.new_game:
				if $ResumeLabel.visible:
					_set_selected_menu(SelectedMenu.resume)
				else:
					if OS.get_name() == "iOS": # quit is hidden on iOS
						_set_selected_menu(SelectedMenu.credits)
					else:
						_set_selected_menu(SelectedMenu.quit)
			SelectedMenu.options:
				_set_selected_menu(SelectedMenu.new_game)
			SelectedMenu.credits:
				_set_selected_menu(SelectedMenu.options)
			SelectedMenu.quit:
				_set_selected_menu(SelectedMenu.credits)
	if _is_menu_down_pressed:
		match _selected_menu:
			SelectedMenu.resume:
				_set_selected_menu(SelectedMenu.new_game)
			SelectedMenu.new_game:
				_set_selected_menu(SelectedMenu.options)
			SelectedMenu.options:
				_set_selected_menu(SelectedMenu.credits)
			SelectedMenu.credits:
				if OS.get_name() == "iOS": # quit is hidden on iOS
					if $ResumeLabel.visible:
						_set_selected_menu(SelectedMenu.resume)
					else:
						_set_selected_menu(SelectedMenu.new_game)
				else:
					_set_selected_menu(SelectedMenu.quit)
			SelectedMenu.quit:
				if $ResumeLabel.visible:
					_set_selected_menu(SelectedMenu.resume)
				else:
					_set_selected_menu(SelectedMenu.new_game)
	if _is_interaction_pressed:
		match _selected_menu:
			SelectedMenu.resume:
				_resume()
			SelectedMenu.new_game:
				_new_game()
			SelectedMenu.credits:
				_display_credits()
			SelectedMenu.options:
				_display_settings()
			SelectedMenu.quit:
				_quit()

func _set_selected_menu(_new_selected_menu: SelectedMenu):
	_selected_menu = _new_selected_menu
	
	var tween := get_tree().create_tween().bind_node($MenuSelector)
	tween.set_parallel().set_trans(Tween.TRANS_QUAD)
	
	match _selected_menu:
		SelectedMenu.resume:
			tween.tween_property($MenuSelector, "global_position", $ResumeLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.new_game:
			tween.tween_property($MenuSelector, "global_position", $NewGameLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.options:
			tween.tween_property($MenuSelector, "global_position", $OptionsLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.credits:
			tween.tween_property($MenuSelector, "global_position", $CreditsLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.quit:
			tween.tween_property($MenuSelector, "global_position", $QuitLabel.get_global_rect().get_center(), 0.1)

func _resume():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.battlefield, SceneTransition.TransitionType.slow_dissolve)

func _new_game():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.run_params)

func _display_credits():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.credits)

func _display_settings():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.settings)

func _quit():
	get_tree().quit()

func _go_to_privacy_policy():
	OS.shell_open("https://fredjul.github.io/TheLastGame-PrivacyPolicy/")
