extends Node

func get_random_enemies() -> Array[Enemy]:
	var enemies: Array[Enemy] = []
	
	var floor_number := Game.state.get_floor_number()
	
	if Game.state.is_newspaper_room():
		var bosses_scenes: Array[PackedScene] = []
		match floor_number:
			2:
				bosses_scenes.append(load("res://src/entities/enemies/floor_2/FamineEnemy.tscn"))
			3:
				bosses_scenes.append(load("res://src/entities/enemies/floor_3/PestilenceEnemy.tscn"))
			4:
				bosses_scenes.append(load("res://src/entities/enemies/floor_4/WarEnemy.tscn"))
		
		var boss_scene_idx := Game.random_gen.randi_range(0, bosses_scenes.size() - 1)
		var boss_enemy := bosses_scenes[boss_scene_idx].instantiate() as Enemy
		
		enemies.append(boss_enemy)
	elif Game.state.is_boss_room():
		var bosses_scenes: Array[PackedScene] = []
		match floor_number:
			1:
				bosses_scenes.append(load("res://src/entities/enemies/floor_1/FairyQueenEnemy.tscn"))
				bosses_scenes.append(load("res://src/entities/enemies/floor_1/WolfEnemy.tscn"))
				bosses_scenes.append(load("res://src/entities/enemies/floor_1/UmberHulkEnemy.tscn"))
			2:
				bosses_scenes.append(load("res://src/entities/enemies/floor_2/GiantToadEnemy.tscn"))
				bosses_scenes.append(load("res://src/entities/enemies/floor_2/MechaGolemEnemy.tscn"))
				bosses_scenes.append(load("res://src/entities/enemies/floor_2/ShieldGuardianEnemy.tscn"))
			3:
				bosses_scenes.append(load("res://src/entities/enemies/floor_3/YetiEnemy.tscn"))
				bosses_scenes.append(load("res://src/entities/enemies/floor_3/FishfolkBruteEnemy.tscn"))
			4:
				bosses_scenes.append(load("res://src/entities/enemies/floor_4/DeathEnemy.tscn"))
			5:
				if Game.state.gods_offended or Game.state.nb_horsemen_killed == 4:
					bosses_scenes.append(load("res://src/entities/enemies/floor_5/ArchangelEnemy.tscn"))
				else:
					bosses_scenes.append(load("res://src/entities/enemies/floor_5/AngelEnemy.tscn"))
		
		var boss_scene_idx := Game.random_gen.randi_range(0, bosses_scenes.size() - 1)
		var boss_enemy := bosses_scenes[boss_scene_idx].instantiate() as Enemy
		
		enemies.append(boss_enemy)
	else:
		var room_number := Game.state.get_room_number()
		
		var enemies_scenes: Array[PackedScene] = []
		match floor_number:
			1:
				enemies_scenes.append(load("res://src/entities/enemies/floor_1/SlimeEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_1/RatfolkDruidEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_1/BloodsilkEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_1/KoboldShamanEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_1/FishfolkShamanEnemy.tscn"))
			2:
				enemies_scenes.append(load("res://src/entities/enemies/floor_2/FoxEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_2/GoblinBoarRiderEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_2/ScorpionEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_2/FungalMonsterEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_2/PumpkinEnemy.tscn"))
			3:
				enemies_scenes.append(load("res://src/entities/enemies/floor_3/BubblemancerEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_3/ElvenWizardessEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_3/FaerieDragonEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_3/GargoyleEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_3/InkbenderEnemy.tscn"))
			4:
				enemies_scenes.append(load("res://src/entities/enemies/floor_4/DemonEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_4/GoblinShamanEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_4/WarlockEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_4/ImpEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_4/PlagueDoctorEnemy.tscn"))
			5:
				enemies_scenes.append(load("res://src/entities/enemies/floor_5/LizardfolkWarriorEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_5/RatfolkRogueEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_5/SpiderEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_5/WingedKoboldEnemy.tscn"))
				enemies_scenes.append(load("res://src/entities/enemies/floor_5/OrcWarlockEnemy.tscn"))
		
		@warning_ignore("integer_division")
		var added_difficulty := room_number / 4
		for i in (3 + added_difficulty):
			var enemy_scene_idx := Game.random_gen.randi_range(0, enemies_scenes.size() - 1)
			enemies.append(enemies_scenes[enemy_scene_idx].instantiate())
	
	return enemies
