class_name GameState
extends Resource

enum DifficultyLevel {easy, medium, hard}
enum FiringAssistance {none, aim, auto}

# Global
@export var total_souls := 0
@export var unlocked_item_keys: Array[String] = []
@export var unlocked_challenge_keys: Array[String] = []
@export var read_messages: Array[String] = []
@export var language_code := ""
@export var nb_machines_bombed := 0
@export var heart_deposit := 0
@export var coin_deposit := 0
@export var difficulty_level := DifficultyLevel.medium
@export var firing_assistance := FiringAssistance.aim
@export var musics_enabled := true

# Run-specific data (that need to be reseted)
@export var run_seed := 0
@export var run_souls := 0
@export var level := 0
@export var room_type := Rooms.RoomType.start
@export var left_door_type := Rooms.RoomType.item
@export var right_door_type := Rooms.RoomType.item
@export var left_door_protection := Rooms.DoorProtection.none
@export var right_door_protection := Rooms.DoorProtection.none
@export var coins := 0
@export var luck_factor := 1.0
@export var nb_bombs := 0
@export var nb_keys := 0
@export var used_bombs := 0
@export var used_keys := 0
@export var heart_bank_machine_broken := false
@export var coin_bank_machine_broken := false
@export var gods_offended := false
@export var initial_players: Array[PlayerState] = []
@export var players: Array[PlayerState] = []
@export var previous_rooms: Array[Rooms.RoomType] = []
@export var nb_horsemen_killed := 0

func get_floor_number() -> int:
	@warning_ignore("integer_division")
	return floori((level - 1) / 10) + 1

func get_room_number() -> int:
	return ((level - 1) % 10) + 1

func is_before_boss_room() -> bool:
	return get_room_number() == 9

func is_boss_room() -> bool:
	return room_type == Rooms.RoomType.boss

func is_newspaper_room() -> bool:
	return room_type == Rooms.RoomType.newspaper

func is_first_final_boss_room() -> bool:
	return level == 40

func is_second_final_boss_room() -> bool:
	return level == 50
