extends AudioStreamPlayer

enum MusicTheme {none, main_menu, dungeon, boss, sad}

var _current_theme := MusicTheme.none

func _ready():
	process_mode = Node.PROCESS_MODE_ALWAYS

func play_main_menu_music():
	play_theme(MusicTheme.main_menu)

func play_dungeon_music():
	play_theme(MusicTheme.dungeon)

func play_boss_music():
	play_theme(MusicTheme.boss)

func play_sad_music():
	play_theme(MusicTheme.sad)

func play_theme(_theme: MusicTheme):
	if not Game.state.musics_enabled or _current_theme == _theme:
		return
	
	_current_theme = _theme
	
	if playing:
		var fade_out_tween := get_tree().create_tween().bind_node(self)
		fade_out_tween.tween_property(self, "volume_db", -40.0, 0.5)
		await fade_out_tween.finished
	
	volume_db = -40.0
	
	var new_volume_db := 0.0
	match _current_theme:
		MusicTheme.main_menu:
			stream = load("res://assets/musics/sawsquarenoise - Tittle Screen.ogg")
			new_volume_db = -20.0
		MusicTheme.dungeon:
			match randi_range(0, 2):
				0:
					stream = load("res://assets/musics/chiptune-grooving.ogg")
					new_volume_db = -17.0
				1:
					stream = load("res://assets/musics/Eggy Toast - The Magician.ogg")
					new_volume_db = -15.0
				2:
					stream = load("res://assets/musics/Eggy Toast - Ghost.ogg")
					new_volume_db = -10.0
		MusicTheme.boss:
			match randi_range(0, 1):
				0:
					stream = load("res://assets/musics/urgencycombat.ogg")
					new_volume_db = -17.0
				1:
					stream = load("res://assets/musics/Eggy Toast - Condemned.ogg")
					new_volume_db = -10.0
		MusicTheme.sad:
			stream = load("res://assets/musics/Metaruka - The End (mix by Avgvst).ogg")
			new_volume_db = -15.0
	play()
	
	var fade_in_tween := get_tree().create_tween().bind_node(self)
	fade_in_tween.tween_property(self, "volume_db", new_volume_db, 0.5)

func stop_immediately():
	if playing:
		_current_theme = MusicTheme.none
		stop()

func stop_with_fade_out():
	if playing:
		_current_theme = MusicTheme.none
		
		var fade_out_tween := get_tree().create_tween().bind_node(self)
		fade_out_tween.tween_property(self, "volume_db", -50.0, 1.0)
		await fade_out_tween.finished
		stop()
