class_name ActivableIndicator
extends Node2D

var _dark_color = Color.from_string("#7a7a7a", Color.WHITE)
var _darker_color = Color.from_string("#7a7a7a6d", Color.WHITE)

func set_controller_number(_controller_number: int):
	$Activable/ActivableButton.set_controller_number(_controller_number)
	$SingleUse/SingleUseButton.set_controller_number(_controller_number)

func set_activable_item(item: GDScript, current_charge: int):
	if item:
		$Activable/ItemContainer.modulate = _dark_color
		$Activable/ChargeContainer.modulate = _dark_color
		$Activable/ChargeBar.visible = true
		$Activable/ActivableButton.modulate = Color.WHITE
		$Activable/Item.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[item])
		
		var max_height := 26.0
		var item_instance := item.new() as BaseItem
		var max_charges := item_instance.needed_charges
		var new_region_rect: Rect2 = $Activable/ChargeBar.region_rect
		if current_charge == max_charges:
			new_region_rect.end.y = max_height
			$Activable/ChargeBar.offset.y = 0
			$Activable/ChargeBar.modulate = Color.WHITE
			$Activable/Item.modulate = Color.WHITE
			$ActivableAnimationPlayer.play("shake")
		elif current_charge == 0:
			new_region_rect.end.y = 0
			$Activable/Item.modulate = _dark_color
			$ActivableAnimationPlayer.stop()
		else:
			var bar_height := max_height / (float(max_charges) / float(current_charge))
			new_region_rect.end.y = bar_height
			$Activable/ChargeBar.offset.y = max_height - bar_height
			$Activable/ChargeBar.modulate = _dark_color
			$Activable/Item.modulate = _dark_color
			$ActivableAnimationPlayer.stop()
		$Activable/ChargeBar.region_rect = new_region_rect
	else:
		$Activable/ItemContainer.modulate = _darker_color
		$Activable/ChargeContainer.modulate = _darker_color
		$Activable/ChargeBar.visible = false
		$Activable/ActivableButton.modulate = _dark_color
		$Activable/Item.texture = null
		$ActivableAnimationPlayer.stop()

func set_single_use_item(item: GDScript):
	if item:
		$SingleUse/SingleUseItemContainer.modulate = _dark_color
		$SingleUse/SingleUseButton.modulate = Color.WHITE
		$SingleUse/SingleUseItem.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[item])
		$SingleUseAnimationPlayer.play("shake")
	else:
		$SingleUse/SingleUseItemContainer.modulate = _darker_color
		$SingleUse/SingleUseButton.modulate = _dark_color
		$SingleUse/SingleUseItem.texture = null
		$SingleUseAnimationPlayer.stop()
