extends RichTextLabel


signal finished_typing()

## The speed with which the text types out
@export var seconds_per_step: float = 0.03

## Automatically have a brief pause when these characters are encountered
@export var pause_at_characters: String = "."

var last_wait_index: int = -1
var last_mutation_index: int = -1
var waiting_seconds: float = 0

var is_typing: bool = false:
	set(value):
		if is_typing != value and value == false:
			finished_typing.emit()
		is_typing = value
	get:
		return is_typing


func _process(delta: float) -> void:
	if self.is_typing:
		# Type out text
		if visible_ratio < 1:
			# See if we are waiting
			if waiting_seconds > 0:
				waiting_seconds = waiting_seconds - delta
			# If we are no longer waiting then keep typing
			if waiting_seconds <= 0:
				_type_next(delta, waiting_seconds)
		else:
			self.is_typing = false


func skip_typing() -> void:
	if self.is_typing and visible_ratio < 1:
		visible_characters = get_total_character_count()
		self.is_typing = false
		finished_typing.emit()

# Start typing out the text
func type_out() -> void:
	visible_characters = 0
	self.is_typing = true
	waiting_seconds = 0
	
	# Text isn't calculated until the next frame
	await get_tree().process_frame
	
	if get_total_character_count() == 0:
		self.is_typing = false
	elif seconds_per_step == 0:
		visible_characters = get_total_character_count()
		self.is_typing = false


# Type out the next character(s)
func _type_next(delta: float, seconds_needed: float) -> void:
	if visible_characters == get_total_character_count():
		return
	
	if last_mutation_index != visible_characters:
		last_mutation_index = visible_characters
	
	var additional_waiting_seconds := 0.0
	if visible_characters > 0 and text[visible_characters -1] in pause_at_characters.split():
		additional_waiting_seconds += seconds_per_step * 10
	
	if last_wait_index != visible_characters and additional_waiting_seconds > 0:
		last_wait_index = visible_characters
		waiting_seconds += additional_waiting_seconds
	else:
		visible_characters += 1
		seconds_needed += seconds_per_step
		if seconds_needed > delta:
			waiting_seconds += seconds_needed
		else:
			_type_next(delta, seconds_needed)
