class_name Door
extends Interactable

@export var door_type := Rooms.RoomType.item : set = _set_door_type
@export var door_protection := Rooms.DoorProtection.none : set = _set_door_protection

func _ready():
	$CollisionShape2D.set_deferred("disabled", true)
	$DoorLabel.hide_button()

func _set_door_type(_door_type: Rooms.RoomType):
	door_type = _door_type
	
	$DoorLabel.text = Rooms.DOOR_DESCRIPTIONS[door_type]

func _set_door_protection(_door_protection: Rooms.DoorProtection):
	door_protection = _door_protection
	($Destroyable as Destroyable).is_destroyed = true # by default it is not destroyable
	
	match door_protection:
		Rooms.DoorProtection.none:
			$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(1, 2))
		Rooms.DoorProtection.key:
			$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(0, 3))
		Rooms.DoorProtection.spikes:
			$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(1, 3))
		Rooms.DoorProtection.rocks:
			($Destroyable as Destroyable).is_destroyed = false
			$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(2, 2))
		Rooms.DoorProtection.boss_key:
			$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(0, 2))

func allow_interaction():
	if visible:
		if door_protection != Rooms.DoorProtection.boss_key:
			$DoorLabel.fade_in(0)
		$CollisionShape2D.set_deferred("disabled", false)

func show_description(_controller_number: int):
	if door_protection != Rooms.DoorProtection.boss_key:
		$DoorLabel.show_button(_controller_number)

func hide_description():
	$DoorLabel.hide_button()

func interact(_player: Player):
	if is_consumed:
		return
	
	if Game.state.read_messages.find("SIGN_FLOOR_1_MSG") == -1:
		EventBus.emit_create_effect(TextEffect.new("READ_SIGN_WARNING", 2.0), _player.global_position)
		return
	
	match door_protection:
		Rooms.DoorProtection.none:
			Sounds.play_door_open()
		Rooms.DoorProtection.key:
			if not RoomUtils.use_key(_player):
				return
			Sounds.play_door_open()
		Rooms.DoorProtection.spikes:
			_player.hurt(1, false, false, false, true)
		Rooms.DoorProtection.rocks:
			if not $Destroyable.is_destroyed:
				EventBus.emit_create_effect(TextEffect.new("BLOCKED_DOOR"), _player.global_position)
				return
		Rooms.DoorProtection.boss_key:
			if not _player.state.has_item(BossKeyItem):
				EventBus.emit_create_effect(TextEffect.new("FINAL_BOSS_DOOR_EXPLANATION", 2.0), _player.global_position)
				return
			_player.remove_item(BossKeyItem)
			Sounds.play_door_open()
	
	is_consumed = true
	EventBus.anim_open_door.emit(self)

func _on_destroyable_destroyed(_player: Player):
	$TileMap.set_cell(0, Vector2i(0, 0), 0, Vector2i(2, 3))
