class_name RoomUtils

static func get_nb_items(players: Array[Player]) -> int:
	if Game.state.room_type == Rooms.RoomType.keys or Game.state.room_type == Rooms.RoomType.bombs or Game.state.room_type == Rooms.RoomType.coins:
		return 1
	
	var nb_alive_players := players.filter(func(p): return p.is_alive()).size()
	match nb_alive_players:
		2:
			return Game.luck_based_randi_range(1, 2)
		3:
			return Game.luck_based_randi_range(1, 3)
		4:
			return Game.luck_based_randi_range(2, 4)
	
	return 1

static func get_random_item(item_pool: Array, upgrade_rarity: int, previous_items: Array[GDScript] = []) -> GDScript:
	if item_pool.size() == 1:
		return item_pool.front()
	elif Game.state.players.any(func(p: PlayerState): return p.has_item(MixItem)):
		item_pool = Items.COMMON_POOL + Items.RARE_POOL + Items.LEGENDARY_POOL
	elif upgrade_rarity == 1 and item_pool == Items.COMMON_POOL:
		item_pool = Items.RARE_POOL
	elif upgrade_rarity >= 2 or (upgrade_rarity == 1 and item_pool == Items.RARE_POOL):
		item_pool = Items.LEGENDARY_POOL
	
	var filtered_item_pool: Array
	for item in item_pool:
		if Items.STATS_POOL.find(item) != -1:
			filtered_item_pool.append(item)
		else:
			var already_obtained := false
			for player in Game.state.players:
				if player.has_item(item):
					already_obtained = true
					break
			
			if not already_obtained:
				filtered_item_pool.append(item)
	
	if filtered_item_pool.is_empty():
		filtered_item_pool = Items.STATS_POOL
	
	if filtered_item_pool.size() == 1:
		return filtered_item_pool.front()
	
	for previous_item in previous_items:
		filtered_item_pool.erase(previous_item) # We avoid generating the same item than before
	
	# We sort items by rarity (lowest rarity first) then we take one randomly with more chances to get a lowest rarity one (but influenced by luck)
	filtered_item_pool.sort_custom(func(a, b): return Items.RARITY_BY_CLASS[a] < Items.RARITY_BY_CLASS[b])
	var item_idx := round((filtered_item_pool.size() - 1) * pow(Game.random_gen.randf(), 1.5/pow(Game.state.luck_factor, 2))) as int
	return filtered_item_pool[item_idx]

static func try_hacking(_player: Player) -> bool:
	if Game.random_gen.randi_range(0, 1) == 1:
		EventBus.emit_create_effect(TextEffect.new("HACKED"), _player.global_position)
		return true
	
	return false

static func use_key(_player: Player) -> bool:
	if Game.state.nb_keys > 0:
		Game.state.nb_keys -= 1
		EventBus.emit_create_effect(TextEffect.new("KEY_USED"), _player.global_position)
		Game.state.used_keys += 1
		EventBus.update_battlefield_ui.emit()
		return true
	else:
		if _player.state.has_item(DarkKeyItem):
			_player.hurt(1, false, true, true, true)
			return true
		else:
			EventBus.emit_create_effect(TextEffect.new("NO_KEY"), _player.global_position)
			return false

static func get_all_message_keys(msg_base_key: String) -> Array[String]:
	var msg_keys: Array[String] = []
	var msg_number := 1
	while true:
		var key := "%s_%s" % [msg_base_key, msg_number]
		if TranslationServer.tr(key) == key:
			break
		
		msg_keys.append(key)
		msg_number += 1
	
	return msg_keys
