extends Node2D

@export var is_coin_atm := false

func _ready():
	$DepositAltar.is_coin_altar = is_coin_atm
	$WithdrawalAltar.is_coin_altar = is_coin_atm
	_update_ui()

func _on_destroyable_destroyed(_player: Player):
	if is_coin_atm:
		Game.state.coin_bank_machine_broken = true
	else:
		Game.state.heart_bank_machine_broken = true
	_update_ui()
	
	if is_coin_atm:
		_player.add_coins(Game.luck_based_randi_range(10, 30))
	else:
		_player.state.fill_all_red_hearts()
		EventBus.emit_create_effect(TextEffect.new("FULL_LIFE"), _player.global_position)
	
	EventBus.update_battlefield_ui.emit()
	EventBus.on_machine_bombed.emit()

func _update_ui():
	if is_coin_atm:
		$Machine/MachineSprite2D.texture = load("res://src/battlefield/rooms/bank_room/atm_machine/coin_bank_machine.png")
		
		if Game.state.coin_bank_machine_broken:
			$DepositAltar.disable()
			$WithdrawalAltar.disable()
			
			$Machine/MachineSprite2D.texture = load("res://src/battlefield/rooms/bank_room/atm_machine/broken_coin_bank_machine.png")
			$Machine/CountLabel.visible = false
			$Machine/Destroyable.is_destroyed = true
		elif Game.state.coin_deposit == 0:
			$Machine/CountLabel.text = ""
		else:
			$Machine/CountLabel.text = str(Game.state.coin_deposit)
	else:
		if Game.state.heart_bank_machine_broken:
			$DepositAltar.disable()
			$WithdrawalAltar.disable()
			
			$Machine/MachineSprite2D.texture = load("res://src/battlefield/rooms/bank_room/atm_machine/broken_heart_bank_machine.png")
			$Machine/CountLabel.visible = false
			$Machine/Destroyable.is_destroyed = true
		elif Game.state.heart_deposit == 0:
			$Machine/CountLabel.text = ""
		else:
			$Machine/CountLabel.text = str(Game.state.heart_deposit)

func _on_hack_failure(_player: Player):
	$Machine/Destroyable.destroy(_player, true)

func _on_deposit_altar_deposit_done():
	_update_ui()

func _on_withdrawal_altar_withdrawal_done():
	_update_ui()
