class_name DepositAltar
extends Interactable

signal deposit_done()
signal hack_failure(_player: Player)

var is_coin_altar := false

var _disabled := false

func _ready():
	$AnimationPlayer.play("appear")

func interact(_player: Player):
	if _disabled:
		return
	
	Sounds.play_interaction()
	
	if _player.state.has_item(HackerItem):
		if RoomUtils.try_hacking(_player):
			if is_coin_altar:
				_deposit_coin()
			else:
				_deposit_heart()
		else:
			hack_failure.emit(_player)
	elif is_coin_altar and Game.state.coins > 0:
		_deposit_coin()
		Game.state.coins -= 1
		EventBus.update_battlefield_ui.emit()
	elif not is_coin_altar and _player.state.hit_points > 0:
		_deposit_heart()
		_player.hurt(2, true, true, true, true, true)
	else:
		if is_coin_altar:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_COINS"), _player.global_position)
		else:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_HEARTS"), _player.global_position)

func _deposit_coin():
	Game.state.coin_deposit += 1
	
	if Game.state.coin_deposit > 99:
		Game.state.coin_deposit = 99
	
	deposit_done.emit()

func _deposit_heart():
	Game.state.heart_deposit += 1
	
	if Game.state.heart_deposit > 99:
		Game.state.heart_deposit = 99
	
	deposit_done.emit()

func show_description(_controller_number: int):
	if not _disabled:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	$DescriptionLabel.fade_out()

func disable():
	_disabled = true
	$DescriptionLabel.fade_out()
