extends Interactable

@onready var _item_pedestal := $ItemPedestal as ItemPedestal

func _ready():
	_item_pedestal.item_pool = Items.COMMON_POOL
	
	$AnimationPlayer.play("appear")

func show_description(_controller_number: int):
	if not $MachineSprite2D/Destroyable.is_destroyed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	$DescriptionLabel.fade_out()

func interact(_player: Player):
	if $MachineSprite2D/Destroyable.is_destroyed:
		return
	
	Sounds.play_interaction()
	
	if _player.state.has_item(HackerItem):
		if RoomUtils.try_hacking(_player):
			_item_pedestal.is_consumed = false
			EventBus.reroll_all_items.emit(-1, null)
		else:
			$MachineSprite2D/Destroyable.destroy(_player, true)
	else:
		if _player.state.hit_points < 2:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_HEARTS"), _player.global_position)
			return
		
		_player.hurt(2, true, true, true, true)
		_item_pedestal.is_consumed = false
		EventBus.reroll_all_items.emit(-1, null)

func _on_destroyable_destroyed(_player: Player):
	$MachineSprite2D.texture = load("res://src/battlefield/rooms/blood_donation_room/broken_blood_donation_machine.png")
	if Game.luck_based_randi_range(0, 4) == 4:
		_item_pedestal.item_pool = [FakeHeartsItem]
	else:
		_item_pedestal.item_pool = [HeartItem]
	_item_pedestal.generate_items(false)
	
	$DescriptionLabel.fade_out(true)
	
	EventBus.on_machine_bombed.emit()
