class_name Chest
extends Interactable

var _item: GDScript
var _is_gold_chest := false
var _secondary_item: GDScript
var _display_secondary_item := false
var _previous_items: Array[GDScript] = []
var _upgrade_rarity := 0

func _ready():
	if Game.luck_based_randi_range(0, 4) == 4:
		_is_gold_chest = true
		$Sprite2D.frame = 18
	
	$AnimationPlayer.play("chest_appear")
	
	EventBus.reroll_all_items.connect(_on_reroll)
	EventBus.altern_items.connect(_on_item_alterned)

func _generate_items(upgrade_rarity: int, force_item_class: GDScript = null):
	_upgrade_rarity += upgrade_rarity
	
	if force_item_class:
		_item = force_item_class
		_secondary_item = force_item_class
	else:
		var item_pool := Items.RARE_POOL
		if _is_gold_chest:
			item_pool = Items.LEGENDARY_POOL
		
		_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
		_secondary_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
	
	_previous_items.append(_item)
	_display_secondary_item = false
	
	_update_item_state()
	
	$AnimationPlayer.play("item_appear")
	await $AnimationPlayer.animation_finished
	$AnimationPlayer.play("item_moving")

func _get_displayed_item() -> GDScript:
	if _display_secondary_item and _secondary_item:
		return _secondary_item
	else:
		return _item

func _update_item_state():
	var displayed_item := _get_displayed_item()
	
	if displayed_item:
		$ItemSprite2D.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[displayed_item])
		$DescriptionLabel.set_item(displayed_item)

func interact(_player: Player):
	if not is_consumed:
		Sounds.play_interaction()
		
		if _item:
			$DescriptionLabel.fade_out(true)
			is_consumed = true
			_player.add_item(_get_displayed_item())
			
			$AnimationPlayer.play("consumed")
		elif RoomUtils.use_key(_player):
			# Once we use a key, do not allow to destroy the chest anymore
			$Sprite2D/Destroyable.queue_free()
			
			is_consumed = true # Avoid interaction while playing anim
			if _is_gold_chest:
				$AnimationPlayer.play("open_gold_chest")
			else:
				$AnimationPlayer.play("open_normal_chest")
			await $AnimationPlayer.animation_finished
			is_consumed = false
			
			if Game.luck_based_randi_range(0, 1) == 1:
				_generate_items(false)
			else:
				$DescriptionLabel.fade_out(true)
				is_consumed = true
				EventBus.emit_create_effect(TextEffect.new("EMPTY_CHEST"), _player.global_position)

func show_description(_controller_number: int):
	if not is_consumed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	if not is_consumed:
		$DescriptionLabel.fade_out()

func _on_destroyable_destroyed(_player: Player):
	_player.add_coins(Game.luck_based_randi_range(10, 30))
	EventBus.update_battlefield_ui.emit()
	queue_free()

func _on_item_alterned():
	if not is_consumed:
		_display_secondary_item = !_display_secondary_item
		_update_item_state()

func _on_reroll(upgrade_rarity: int, force_item_class: GDScript):
	if not is_consumed and _item:
		_generate_items(upgrade_rarity, force_item_class)
