class_name ItemPedestal
extends Interactable

var item_pool := []
var is_locked := false
var unlock_souls_price := 0
var unlock_challenge := ""

var _item: GDScript
var _secondary_item: GDScript
var _display_secondary_item := false
var _previous_items: Array[GDScript] = []
var _upgrade_rarity := 0

func _ready():
	if not item_pool.is_empty():
		generate_items(false)
	else:
		is_consumed = true
		$AnimationPlayer.play("appear")
		await $AnimationPlayer.animation_finished
		$AnimationPlayer.play("consumed")
	
	EventBus.reroll_all_items.connect(_on_reroll)
	EventBus.on_item_unlocked.connect(_on_item_unlocked)
	EventBus.altern_items.connect(_on_item_alterned)

func is_interactable() -> bool:
	return not is_consumed and (not is_locked or unlock_souls_price > 0)

func generate_items(upgrade_rarity: int, force_item_class: GDScript = null):
	if upgrade_rarity == -1:
		_upgrade_rarity = 0 # avoid always generating good objects with blood donation
	else:
		_upgrade_rarity += upgrade_rarity
	
	if item_pool.size() > 1 and force_item_class:
		_item = force_item_class
		_secondary_item = force_item_class
	else:
		_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
		_secondary_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
	
	_previous_items.append(_item)
	_display_secondary_item = false
	is_consumed = false
	
	_update_item_state()
	
	$AnimationPlayer.play("appear")
	await $AnimationPlayer.animation_finished
	$AnimationPlayer.play("not_consumed")

func _get_displayed_item() -> GDScript:
	if _display_secondary_item and _secondary_item:
		return _secondary_item
	else:
		return _item

func _update_item_state():
	var displayed_item := _get_displayed_item()
	
	if displayed_item:
		$ItemSprite2D.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[displayed_item])
		
		_update_lock_state(displayed_item)

func _update_lock_state(displayed_item: GDScript):
	if is_locked and unlock_souls_price > 0:
		is_locked = Game.state.unlocked_item_keys.find(_get_level_item_key()) == -1
	elif is_locked and not unlock_challenge.is_empty():
		is_locked = Game.state.unlocked_challenge_keys.find(unlock_challenge) == -1
	
	if is_locked and unlock_souls_price > 0:
		$DescriptionLabel.text = "SPEND_SOULS_TO_UNLOCK"
	elif is_locked and not unlock_challenge.is_empty():
		$DescriptionLabel.text = unlock_challenge
	else:
		$DescriptionLabel.set_item(displayed_item)
	
	if is_locked:
		$Lock.visible = true
		$Lock/PriceLabel.text = str(unlock_souls_price)
		
		if not unlock_challenge.is_empty():
			$Lock/SoulsSprite2D.visible = false
			$Lock/PriceLabel.visible = false
	else:
		$Lock.visible = false

func interact(_player: Player):
	Sounds.play_interaction()
	if is_locked and unlock_souls_price > 0:
		if Game.state.total_souls >= unlock_souls_price:
			is_locked = false
			Game.state.unlocked_item_keys.append(_get_level_item_key())
			Game.state.total_souls -= unlock_souls_price
			EventBus.update_battlefield_ui.emit()
			
			_update_lock_state(_item)
		else:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_SOULS"), _player.global_position)
	elif not is_locked and not is_consumed:
		$DescriptionLabel.fade_out(true)
		
		$AnimationPlayer.play("consumed")
		is_consumed = true
		
		_player.add_item(_get_displayed_item())

func show_description(_controller_number: int):
	if is_locked and not unlock_challenge.is_empty():
		$DescriptionLabel.hide_button()
	else:
		$DescriptionLabel.show_button(_controller_number)
		
	if not is_consumed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	if not is_consumed:
		$DescriptionLabel.fade_out()

func _get_level_item_key() -> String:
	return "FLOOR_%d_ITEM_%d" % [Game.state.get_floor_number(), Items.ITEM_IDS_BY_CLASS[_item]]

func _on_item_unlocked():
	_update_lock_state(_item)

func _on_item_alterned():
	if not is_consumed:
		_display_secondary_item = !_display_secondary_item
		_update_item_state()
	
func _on_reroll(upgrade_rarity: int, force_item_class: GDScript):
	if not is_consumed:
		generate_items(upgrade_rarity, force_item_class)
