class_name LaboratoryItem
extends Interactable


var item_pool := []

var _item: GDScript
var _secondary_item: GDScript
var _display_secondary_item := false
var _previous_items: Array[GDScript] = []
var _keys_price := 2
var _bombs_price := 0
var _upgrade_rarity := 0

func _ready():
	if Game.random_gen.randi_range(0, 1) == 1:
		_keys_price = 0
		_bombs_price = 2
		$KeySprite2D.visible = false
		$BombSprite2D.visible = true
	
	_generate_items(false)
	
	EventBus.reroll_all_items.connect(_on_reroll)
	EventBus.altern_items.connect(_on_item_alterned)
	EventBus.discount_items.connect(_on_discount_items)

func _generate_items(upgrade_rarity: int, force_item_class: GDScript = null):
	_upgrade_rarity += upgrade_rarity
	
	if force_item_class:
		_item = force_item_class
		_secondary_item = force_item_class
	else:
		_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
		_secondary_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
	
	_previous_items.append(_item)
	_display_secondary_item = false
	
	_update_item_state()
	
	$AnimationPlayer.play("appear")
	await $AnimationPlayer.animation_finished
	$AnimationPlayer.play("not_consumed")
	is_consumed = false

func _get_displayed_item() -> GDScript:
	if _display_secondary_item and _secondary_item:
		return _secondary_item
	else:
		return _item

func _update_item_state():
	var displayed_item := _get_displayed_item()
	
	if displayed_item:
		$ItemSprite2D.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[displayed_item])
		$DescriptionLabel.set_item(displayed_item)

func interact(_player: Player):
	if not is_consumed:
		Sounds.play_interaction()
		
		if _keys_price > 0 and Game.state.nb_keys < _keys_price:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_KEYS"), _player.global_position)
			return
		if _bombs_price > 0 and Game.state.nb_bombs < _bombs_price:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_BOMBS"), _player.global_position)
			return
		
		$DescriptionLabel.fade_out(true)
		$AnimationPlayer.play("consumed")
		is_consumed = true
		
		Game.state.nb_keys -= _keys_price
		Game.state.nb_bombs -= _bombs_price
		EventBus.update_battlefield_ui.emit()

func show_description(_controller_number: int):
	if not is_consumed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	if not is_consumed:
		$DescriptionLabel.fade_out()

func _on_item_alterned():
	if not is_consumed:
		_display_secondary_item = !_display_secondary_item
		_update_item_state()

func _on_reroll(upgrade_rarity: int, force_item_class: GDScript):
	if not is_consumed:
		_generate_items(upgrade_rarity, force_item_class)

func _on_discount_items():
	if _keys_price > 0:
		_keys_price = 1
	if _bombs_price > 0:
		_bombs_price = 1
	
	$SecondaryLabel.text = "x1"
