extends Interactable

@onready var _item_pedestal := $ItemPedestal as ItemPedestal

func _ready():
	_item_pedestal.item_pool = Items.COMMON_POOL
	
	$AnimationPlayer.play("appear")

func show_description(_controller_number: int):
	if not is_consumed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	$DescriptionLabel.fade_out()

func interact(_player: Player):
	if is_consumed:
		return
	
	Sounds.play_interaction()
	
	var goods_value := _get_goods_value()
	if goods_value == 0:
		EventBus.emit_create_effect(TextEffect.new("NO_GOODS", 1.2), _player.global_position)
		return
	
	Game.state.coins = 0
	Game.state.nb_keys = 0
	Game.state.nb_bombs = 0
	EventBus.emit_create_effect(TextEffect.new("GIVE_GOODS", 1.6), _player.global_position)
	EventBus.update_battlefield_ui.emit()
	Game.persist_state()
	
	is_consumed = true
	await get_tree().create_timer(1.5, false).timeout	
	is_consumed = false
	
	_item_pedestal.is_consumed = false
	if goods_value < 70:
		EventBus.emit_create_effect(TextEffect.new("OFFERING_COMMON_REWARD", 1.6), $StatueSprite2D.global_position)
		_item_pedestal.item_pool = Items.COMMON_POOL
	elif goods_value < 140:
		EventBus.emit_create_effect(TextEffect.new("OFFERING_RARE_REWARD", 1.6), $StatueSprite2D.global_position)
		_item_pedestal.item_pool = Items.RARE_POOL
	else:
		EventBus.emit_create_effect(TextEffect.new("OFFERING_LEGENDARY_REWARD", 1.6), $StatueSprite2D.global_position)
		_item_pedestal.item_pool = Items.LEGENDARY_POOL
	EventBus.reroll_all_items.emit(0, null)

func _get_goods_value() -> int:
	return Game.state.coins + (15 * Game.state.nb_keys) + (15 * Game.state.nb_bombs)

func _on_destroyable_destroyed(_player: Player):
	is_consumed = true
	$StatueSprite2D.texture = load("res://src/battlefield/rooms/offering_room/blood_statue.png")
	EventBus.emit_create_effect(TextEffect.new("GODS_OFFENDED"), _player.global_position)
	Game.state.gods_offended = true
	
	$DescriptionLabel.fade_out(true)
