class_name ShopItem
extends Interactable

var item_pool := []
var price := 0
var infinite := false

var _item: GDScript
var _secondary_item: GDScript
var _display_secondary_item := false
var _previous_items: Array[GDScript] = []
var _upgrade_rarity := 0

func _ready():
	$PriceLabel.text = str(price)
	_generate_items(false)
	
	EventBus.reroll_all_items.connect(_on_reroll)
	EventBus.altern_items.connect(_on_item_alterned)
	EventBus.discount_items.connect(_on_discount_items)

func _generate_items(upgrade_rarity: int, force_item_class: GDScript = null):
	_upgrade_rarity += upgrade_rarity
	
	if item_pool.size() > 1 and force_item_class:
		_item = force_item_class
		_secondary_item = force_item_class
	else:
		_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
		_secondary_item = RoomUtils.get_random_item(item_pool, _upgrade_rarity, _previous_items)
	
	_previous_items.append(_item)
	_display_secondary_item = false
	
	_update_item_state()
	
	$AnimationPlayer.play("appear")
	await $AnimationPlayer.animation_finished
	$AnimationPlayer.play("not_consumed")
	is_consumed = false

func _get_displayed_item() -> GDScript:
	if _display_secondary_item and _secondary_item:
		return _secondary_item
	else:
		return _item

func _update_item_state():
	var displayed_item := _get_displayed_item()
	
	if displayed_item:
		$ItemSprite2D.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[displayed_item])
		$DescriptionLabel.set_item(displayed_item)

func interact(_player: Player):
	if not is_consumed:
		Sounds.play_interaction()
		
		if Game.state.coins < price:
			EventBus.emit_create_effect(TextEffect.new("NOT_ENOUGH_COINS"), _player.global_position)
			return
	
		if not infinite:
			$DescriptionLabel.fade_out(true)
			$AnimationPlayer.play("consumed")
			is_consumed = true
		
		Game.state.coins -= price
		_player.add_item(_get_displayed_item())
		EventBus.emit_create_effect(TextEffect.new("-%s %s" % [price, tr("COINS")]), _player.global_position)

func show_description(_controller_number: int):
	if not is_consumed:
		$DescriptionLabel.fade_in(_controller_number)

func hide_description():
	if not is_consumed:
		$DescriptionLabel.fade_out()

func _on_item_alterned():
	if not is_consumed:
		_display_secondary_item = !_display_secondary_item
		_update_item_state()

func _on_reroll(upgrade_rarity: int, force_item_class: GDScript):
	if not is_consumed:
		_generate_items(upgrade_rarity, force_item_class)

func _on_discount_items():
	price /= 2
	$PriceLabel.text = str(price)
