class_name StartRoom
extends Node2D

var _debug_items := [LandmineItem,FlailAssistItem,ExplodingSquirrelAssistItem]
var _level_shortcuts := [10, 20, 31]

static var unlockables_by_floor := {
	1: [
		[ShieldItem, 200],
		[HeartReceptacleItem, 500],
		[DamagePotionItem, 900],
		[SingleUseRerollItem, 1500],
		[FireRateWhenIdleItem, null],
		[GiftItem, null],
	],
	2: [
		[ShieldItem, 500],
		[HeartReceptacleItem, 800],
		[FireRatePotionItem, 1500],
		[SingleUseRerollDoorsItem, 2000],
		[DarkKeyItem, null],
		[HealItem, null],
	],
	3: [
		[ShieldItem, 800],
		[HeartReceptacleItem, 1500],
		[LuckPotionItem, 2000],
		[SingleUseRerollItem, 2500],
		[ReviveItem, null],
		[AllStatsUpItem, null],
	],
	4: [
		[ShieldItem, 1000],
		[HeartReceptacleItem, 2000],
		[RangePotionItem, 3000],
		[BiggerBombItem, null],
		[HeartReceptacleConverterItem, null],
		[BossKeyItem, null],
	],
	5: [
		[ShieldItem, 1500],
		[HeartReceptacleItem, 3000],
		[CriticalChancePotionItem, 4000],
		[SingleUseRerollItem, 5000],
		[SingleUseRerollDoorsItem, 5000],
		[SingleUseHomingItem, 5000],
	],
}

func _ready():
	# Populate unlockables
	var unlockables: Array
	if unlockables_by_floor.has(Game.state.get_floor_number()):
		unlockables = unlockables_by_floor[Game.state.get_floor_number()]
	else:
		unlockables = unlockables_by_floor.values().back()
	
	for i in unlockables.size():
		var pedestal := $Pedestals.get_child(i) as ItemPedestal
		pedestal.is_locked = true
		if unlockables[i][1] == null:
			pedestal.unlock_challenge = Items.LOCKED_ITEMS[unlockables[i][0]]
		else:
			pedestal.unlock_souls_price = unlockables[i][1]
		pedestal.item_pool = [unlockables[i][0]]
		pedestal.generate_items(false)
	
	if OS.is_debug_build() and OS.get_name() == "macOS": # Due to crashes, we need to build in debug mode for iOS, not sure why
		# Populate debug items
		var item_pedestal_reference := load("res://src/battlefield/rooms/generic_item_room/item_pedestal/ItemPedestal.tscn") as PackedScene
		var initial_pos := -150
		for debug_item in _debug_items:
			var item_pedestal = item_pedestal_reference.instantiate() as ItemPedestal
			item_pedestal.item_pool = [debug_item]
			item_pedestal.position = Vector2(initial_pos, 150)
			add_child(item_pedestal)
			initial_pos += 150
		
		# Populate ladders
		var ladder_reference := load("res://src/battlefield/objects/ladder/Ladder.tscn") as PackedScene
		var initial_ladder_pos := -150
		for level in _level_shortcuts:
			var ladder = ladder_reference.instantiate() as Ladder
			ladder.set_level(level)
			ladder.position = Vector2(initial_ladder_pos, -120)
			add_child(ladder)
			initial_ladder_pos += 150
