extends BaseMenuScreen

enum SelectedMenu {resume, quit}

var _selected_menu := SelectedMenu.resume

func _ready():
	$UI.visible = false
	
	_update_touch_interface()
	Input.joy_connection_changed.connect(_joy_connection_changed)

func _joy_connection_changed(_device: int, _connected: bool):
	_update_touch_interface()
	
	if not _connected:
		_pause() # pause the game if we disconnected a controller

func _update_touch_interface():
	$UI/MenuSelector.visible = not InputUtils.is_in_touch_mode()
	$UI/ControllerIndication.visible = not InputUtils.is_in_touch_mode() 

func _unhandled_input(_event: InputEvent):
	super(_event) # needed to detect menu actions
	
	if _event.is_action_pressed("pause"):
		if not get_tree().paused and not $UI.visible:
			_pause()
			get_viewport().set_input_as_handled()
			return
	elif not $UI.visible:
		return
	
	# When there is the pause menu, we handle all inputs
	get_viewport().set_input_as_handled()
	
	if _is_menu_up_pressed:
		match _selected_menu:
			SelectedMenu.resume:
				_set_selected_menu(SelectedMenu.quit)
			SelectedMenu.quit:
				_set_selected_menu(SelectedMenu.resume)
	if _is_menu_down_pressed:
		match _selected_menu:
			SelectedMenu.resume:
				_set_selected_menu(SelectedMenu.quit)
			SelectedMenu.quit:
				_set_selected_menu(SelectedMenu.resume)
	if _event.is_action_pressed("pause"):
		if $UI.visible:
			_resume()
	if _is_interaction_pressed:
		if $UI.visible:
			match _selected_menu:
				SelectedMenu.resume:
					_resume()
				SelectedMenu.quit:
					_quit()

func _pause():
	Sounds.play_interaction()
	$UI/MenuSelector.global_position = $UI/ResumeLabel.get_global_rect().get_center()
	_selected_menu = SelectedMenu.resume
	
	get_tree().paused = true
	$UI.visible = true
	$Battlefield.modulate = Color.DARK_GRAY.darkened(0.7)

func _resume():
	Sounds.play_interaction()
	$UI.visible = false
	$Battlefield.modulate = Color.WHITE
	get_tree().paused = false

func _quit():
	Sounds.play_interaction()
	# Reset the state but does not persist the state or quit the run
	Game.load_state()
	
	SceneTransition.change_scene(SceneTransition.main_menu, SceneTransition.TransitionType.slow_dissolve)

func _set_selected_menu(_new_selected_menu: SelectedMenu):
	_selected_menu = _new_selected_menu
	
	var tween := get_tree().create_tween().bind_node($UI/MenuSelector)
	tween.set_parallel().set_trans(Tween.TRANS_QUAD)
	
	match _selected_menu:
		SelectedMenu.resume:
			tween.tween_property($UI/MenuSelector, "global_position", $UI/ResumeLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.quit:
			tween.tween_property($UI/MenuSelector, "global_position", $UI/QuitLabel.get_global_rect().get_center(), 0.1)
