extends Node2D


func _ready():
	var nb_controller := InputUtils.get_real_joypads().size()
	if nb_controller < 4:
		$P4CharacterSelection.enabled = false
	if nb_controller < 3:
		$P3CharacterSelection.enabled = false
	if nb_controller < 2:
		$P2CharacterSelection.enabled = false
	
	_on_character_color_changed()

func _input(_event):
	if _event.is_action_pressed("p1_interaction") and not $P1CharacterSelection.enabled:
		$P1CharacterSelection.enabled = true
		_on_enable_change()
	elif _event.is_action_pressed("p2_interaction") and not $P2CharacterSelection.enabled:
		$P2CharacterSelection.enabled = true
		_on_enable_change()
	elif _event.is_action_pressed("p3_interaction") and not $P3CharacterSelection.enabled:
		$P3CharacterSelection.enabled = true
		_on_enable_change()
	elif _event.is_action_pressed("p4_interaction") and not $P4CharacterSelection.enabled:
		$P4CharacterSelection.enabled = true
		_on_enable_change()
	elif _event.is_action_pressed("p1_cancel"):
		if $P1CharacterSelection.enabled:
			$P1CharacterSelection.enabled = false
			_on_enable_change()
		else:
			_back()
	elif _event.is_action_pressed("p2_cancel"):
		if $P2CharacterSelection.enabled:
			$P2CharacterSelection.enabled = false
			_on_enable_change()
		else:
			_back()
	elif _event.is_action_pressed("p3_cancel"):
		if $P3CharacterSelection.enabled:
			$P3CharacterSelection.enabled = false
			_on_enable_change()
		else:
			_back()
	elif _event.is_action_pressed("p4_cancel"):
		if $P4CharacterSelection.enabled:
			$P4CharacterSelection.enabled = false
			_on_enable_change()
		else:
			_back()
	elif _event.is_action_pressed("start"):
		_start()

func _get_character_selection_widgets() -> Array[CharacterSelection]:
	return [$P1CharacterSelection as CharacterSelection, $P2CharacterSelection as CharacterSelection, $P3CharacterSelection as CharacterSelection, $P4CharacterSelection as CharacterSelection]

func _get_used_colors(ignore_controller_number: int) -> Array[int]:
	var used_colors: Array[int] = []
	
	for character_selection in _get_character_selection_widgets():
		if character_selection.controller_number != ignore_controller_number and character_selection.enabled:
			used_colors.append(character_selection.color_number)
	
	return used_colors

func _get_free_colors(ignore_controller_number: int) -> Array[int]:
	var free_colors: Array[int] = [1, 2, 3, 4, 5]
	for used_color in _get_used_colors(ignore_controller_number):
		free_colors.erase(used_color)
	return free_colors

func _on_character_color_changed():
	for character_selection in _get_character_selection_widgets():
		character_selection.available_color_numbers = _get_free_colors(character_selection.controller_number)

func _on_enable_change():
	Sounds.play_interaction()
	
	var can_start := false
	for character_selection in _get_character_selection_widgets():
		if character_selection.enabled:
			can_start = true
	
	$StartButton.visible = can_start

func _back():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.run_params)

func _start():
	var player_states: Array[PlayerState] = []
	
	var has_still_locked_item := false
	for character_selection in _get_character_selection_widgets():
		if character_selection.enabled and character_selection.item != null:
			var unlock_challenge := Items.LOCKED_ITEMS[character_selection.item] as String
			var is_locked := Game.state.unlocked_challenge_keys.find(unlock_challenge) == -1
			if is_locked:
				character_selection.shake()
				has_still_locked_item = true
	
	if has_still_locked_item:
		Sounds.play_error()
	else:
		for character_selection in _get_character_selection_widgets():
			if character_selection.enabled:
				var player_state := PlayerState.new()
				player_state.controller_number = character_selection.controller_number
				player_state.color_number = character_selection.color_number
				
				if character_selection.item != null:
					var item: BaseItem = character_selection.item.new()
					item.on_obtain(player_state)
					player_state.item_ids.append(Items.ITEM_IDS_BY_CLASS[character_selection.item])
				
				player_states.append(player_state)
		
		if not player_states.is_empty():
			Sounds.play_interaction()
			Game.start_new_run(player_states)
			
			SceneTransition.change_scene(SceneTransition.battlefield, SceneTransition.TransitionType.slow_dissolve)
		else:
			Sounds.play_error()
