class_name CharacterSelection
extends BaseMenuScreen

static var STARTING_ITEMS := [
	NoHeartItem,
	QuadShotItem,
	HackerItem,
	SummonerItem,
	InfiniteBombsItem,
	KnightItem,
	TheChallengeItem,
	]

enum SelectedMenu {color, item}

signal on_color_changed()

@export var controller_number := 1
var color_number := 1
var available_color_numbers := [1] : set = _set_available_color_numbers
var item: GDScript
var sorted_available_items := [null]

var enabled := true : set = _set_enabled

var _selected_menu := SelectedMenu.color

func _ready():
	_accepted_controller_number = [controller_number]
	
	$ControllerNumberLabel.text = "%s %s" % [tr("PLAYER"), str(controller_number)]
	if controller_number != 1:
		color_number = controller_number
		_update_character_sprite()
	
	# We first display unlocked items, then the locked ones
	for starting_item in STARTING_ITEMS:
		var unlock_challenge := Items.LOCKED_ITEMS[starting_item] as String
		var is_locked := Game.state.unlocked_challenge_keys.find(unlock_challenge) == -1
		if not is_locked:
			sorted_available_items.append(starting_item)
	for starting_item in STARTING_ITEMS:
		var unlock_challenge := Items.LOCKED_ITEMS[starting_item] as String
		var is_locked := Game.state.unlocked_challenge_keys.find(unlock_challenge) == -1
		if is_locked:
			sorted_available_items.append(starting_item)
	
	_update_item_sprite()
	_update_touch_interface()
	Input.joy_connection_changed.connect(_joy_connection_changed)

func _joy_connection_changed(_device: int, _connected: bool):
	_update_touch_interface()

func _update_touch_interface():
	if controller_number != 1 and controller_number > InputUtils.get_real_joypads().size():
		$JoinLabel.text = "ADD_CONTROLLER_TO_JOIN"
	else:
		$JoinLabel.text = "VALIDATE_TO_JOIN"
	
	$ArrowSelector2.visible = enabled and InputUtils.is_in_touch_mode()
	
	if InputUtils.is_in_touch_mode():
		_set_selected_menu(SelectedMenu.color)

func _unhandled_input(_event: InputEvent):
	super(_event) # needed to detect menu actions
	
	if _is_menu_up_pressed:
		if _selected_menu == SelectedMenu.item:
			_set_selected_menu(SelectedMenu.color)
	if _is_menu_down_pressed:
		if _selected_menu == SelectedMenu.color:
			_set_selected_menu(SelectedMenu.item)
	if _is_menu_left_pressed:
		if _selected_menu == SelectedMenu.color:
			_select_left_color()
		else:
			_select_left_item()
	if _is_menu_right_pressed:
		if _selected_menu == SelectedMenu.color:
			_select_right_color()
		else:
			_select_right_item()

func _set_selected_menu(_new_selected_menu: SelectedMenu):
	_selected_menu = _new_selected_menu
	
	var tween := get_tree().create_tween().bind_node($ArrowSelector)
	tween.set_parallel().set_trans(Tween.TRANS_QUAD)
	
	match _selected_menu:
		SelectedMenu.color:
			tween.tween_property($ArrowSelector, "global_position", $CharacterSprite2D.global_position, 0.1)
		SelectedMenu.item:
			tween.tween_property($ArrowSelector, "global_position", $ItemSprite2D.global_position, 0.1)

func _select_left_color():
	Sounds.play_interaction()
	
	var current_color_idx := available_color_numbers.find(color_number)
	if current_color_idx == 0:
		color_number = available_color_numbers.back()
	else:
		color_number = available_color_numbers[current_color_idx - 1]
	
	on_color_changed.emit()
	_update_character_sprite()

func _select_right_color():
	Sounds.play_interaction()
	
	var current_color_idx := available_color_numbers.find(color_number)
	if current_color_idx == available_color_numbers.size() - 1:
		color_number = available_color_numbers.front()
	else:
		color_number = available_color_numbers[current_color_idx + 1]
	
	on_color_changed.emit()
	_update_character_sprite()

func _select_left_item():
	Sounds.play_interaction()
	
	var current_item_idx := sorted_available_items.find(item)
	if current_item_idx == 0:
		item = sorted_available_items.back()
	else:
		item = sorted_available_items[current_item_idx - 1]
	
	_update_item_sprite()

func _select_right_item():
	Sounds.play_interaction()
	
	var current_item_idx := sorted_available_items.find(item)
	if current_item_idx == sorted_available_items.size() - 1:
		item = sorted_available_items.front()
	else:
		item = sorted_available_items[current_item_idx + 1]
	
	_update_item_sprite()

func _set_enabled(_enabled: bool):
	enabled = _enabled
	$CharacterSprite2D.visible = _enabled
	$ArrowSelector.visible = _enabled
	$ArrowSelector2.visible = _enabled and InputUtils.is_in_touch_mode()
	$ItemSprite2D.visible = _enabled
	$ItemDescriptionLabel.visible = _enabled
	$JoinLabel.visible = !_enabled
	
	on_color_changed.emit()

func _set_available_color_numbers(_available_color_numbers: Array[int]):
	available_color_numbers = _available_color_numbers
	
	if _available_color_numbers.find(color_number) == -1:
		color_number = _available_color_numbers.front()
		_update_character_sprite()

func _update_character_sprite():
	$CharacterSprite2D.texture = load("res://src/entities/player/character_variant_%s.png" % color_number)

func _update_item_sprite():
	if item == null:
		$ItemSprite2D.texture = load("res://src/menus/player_selection/character_selection/default_starting_item.png")
		$ItemDescriptionLabel.text = "DEFAULT_STARTING_ITEM"
	else:
		var unlock_challenge := Items.LOCKED_ITEMS[item] as String
		var is_locked := Game.state.unlocked_challenge_keys.find(unlock_challenge) == -1
		if is_locked:
			$ItemSprite2D.texture = load("res://src/battlefield/rooms/generic_item_room/item_pedestal/padlock.png")
			$ItemDescriptionLabel.text = unlock_challenge
		else:
			$ItemSprite2D.texture = load("res://assets/textures/items/item_%s.png" % Items.ITEM_IDS_BY_CLASS[item])
			$ItemDescriptionLabel.text = "ITEM_%s" % Items.ITEM_IDS_BY_CLASS[item]

func shake():
	$AnimationPlayer.play("shake")
