extends BaseMenuScreen

enum SelectedMenu {difficulty, firing_assistance}

var _selected_menu := SelectedMenu.difficulty

func _ready():
	_update_difficulty_text()
	_update_firing_assistance_text()
	
	_update_touch_interface()
	Input.joy_connection_changed.connect(_joy_connection_changed)

func _joy_connection_changed(_device: int, _connected: bool):
	_update_touch_interface()

func _update_touch_interface():
	var is_in_touch_mode := InputUtils.is_in_touch_mode()
	
	$FiringAssistanceCatLabel.visible = not is_in_touch_mode
	$FiringAssistanceLabel.visible = not is_in_touch_mode
	$MenuSelector.visible = not is_in_touch_mode
	$TouchArrowSelector.visible = is_in_touch_mode

func _unhandled_input(_event: InputEvent):
	super(_event) # needed to detect menu actions
	
	if _is_menu_up_pressed:
		match _selected_menu:
			SelectedMenu.difficulty:
				_set_selected_menu(SelectedMenu.firing_assistance)
			SelectedMenu.firing_assistance:
				_set_selected_menu(SelectedMenu.difficulty)
	if _is_menu_down_pressed:
		match _selected_menu:
			SelectedMenu.difficulty:
				_set_selected_menu(SelectedMenu.firing_assistance)
			SelectedMenu.firing_assistance:
				_set_selected_menu(SelectedMenu.difficulty)
	if _is_menu_right_pressed or _is_menu_left_pressed:
		match _selected_menu:
			SelectedMenu.difficulty:
				_switch_difficulty(_is_menu_right_pressed)
			SelectedMenu.firing_assistance:
				_switch_firing_assistance(_is_menu_right_pressed)
	
	if _is_cancel_pressed:
		_back()
	if _event.is_action_pressed("start"):
		_start()

func _back():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.main_menu)

func _switch_difficulty(right_sens := true):
	Sounds.play_interaction()
	if right_sens:
		match Game.state.difficulty_level:
			GameState.DifficultyLevel.easy:
				Game.state.difficulty_level = GameState.DifficultyLevel.medium
			GameState.DifficultyLevel.medium:
				Game.state.difficulty_level = GameState.DifficultyLevel.hard
			GameState.DifficultyLevel.hard:
				Game.state.difficulty_level = GameState.DifficultyLevel.easy
	else:
		match Game.state.difficulty_level:
			GameState.DifficultyLevel.easy:
				Game.state.difficulty_level = GameState.DifficultyLevel.hard
			GameState.DifficultyLevel.medium:
				Game.state.difficulty_level = GameState.DifficultyLevel.easy
			GameState.DifficultyLevel.hard:
				Game.state.difficulty_level = GameState.DifficultyLevel.medium
	
	_update_difficulty_text()

func _switch_firing_assistance(right_sens := true):
	Sounds.play_interaction()
	if right_sens:
		match Game.state.firing_assistance:
			GameState.FiringAssistance.none:
				Game.state.firing_assistance = GameState.FiringAssistance.aim
			GameState.FiringAssistance.aim:
				Game.state.firing_assistance = GameState.FiringAssistance.auto
			GameState.FiringAssistance.auto:
				Game.state.firing_assistance = GameState.FiringAssistance.none
	else:
		match Game.state.firing_assistance:
			GameState.FiringAssistance.none:
				Game.state.firing_assistance = GameState.FiringAssistance.auto
			GameState.FiringAssistance.aim:
				Game.state.firing_assistance = GameState.FiringAssistance.none
			GameState.FiringAssistance.auto:
				Game.state.firing_assistance = GameState.FiringAssistance.aim
	
	_update_firing_assistance_text()

func _set_selected_menu(_new_selected_menu: SelectedMenu):
	_selected_menu = _new_selected_menu
	
	var tween := get_tree().create_tween().bind_node($MenuSelector)
	tween.set_parallel().set_trans(Tween.TRANS_QUAD)
	
	match _selected_menu:
		SelectedMenu.difficulty:
			tween.tween_property($MenuSelector, "global_position", $DifficultyLabel.get_global_rect().get_center(), 0.1)
		SelectedMenu.firing_assistance:
			tween.tween_property($MenuSelector, "global_position", $FiringAssistanceLabel.get_global_rect().get_center(), 0.1)

func _update_difficulty_text():
	match Game.state.difficulty_level:
		GameState.DifficultyLevel.easy:
			$DifficultyLabel.text = "EASY_DIFFICULTY"
		GameState.DifficultyLevel.medium:
			$DifficultyLabel.text = "MEDIUM_DIFFICULTY"
		GameState.DifficultyLevel.hard:
			$DifficultyLabel.text = "HARD_DIFFICULTY"

func _update_firing_assistance_text():
	match Game.state.firing_assistance:
		GameState.FiringAssistance.none:
			$FiringAssistanceLabel.text = "FIRING_ASSISTANCE_NONE"
		GameState.FiringAssistance.aim:
			$FiringAssistanceLabel.text = "FIRING_ASSISTANCE_AIM"
		GameState.FiringAssistance.auto:
			$FiringAssistanceLabel.text = "FIRING_ASSISTANCE_AUTO"

func _on_back_button_on_pressed():
	Sounds.play_interaction()
	$ResetConfirmationDialog.visible = false

func _start():
	Sounds.play_interaction()
	SceneTransition.change_scene(SceneTransition.player_selection)

func _select_left_difficulty():
	_switch_difficulty(false)

func _select_right_difficulty():
	_switch_difficulty()
