class_name InputUtils

enum JoypadType {
	TOUCH,
	KEYBOARD,
	XBOX,
	PLAYSTATION,
	NINTENDO,
	UNINDENTIFIED,
}

static func get_real_joypads() -> Array[int]:
	var result: Array[int] = []
	for id in Input.get_connected_joypads():
		var name := Input.get_joy_name(id)
		# hack to bypass https://github.com/godotengine/godot/issues/47656
		if not name == "uinput-fpc" and not name == "uinput-goodix":
			result.append(id)
	
	return result

static func get_type_for(_controller_number: int) -> JoypadType:
	if is_in_touch_mode():
		return JoypadType.TOUCH
	
	var joypad_id := get_joypad_id(_controller_number)
	if joypad_id == -1:
		return JoypadType.KEYBOARD
	
	var joypad_name := Input.get_joy_name(joypad_id)
	var type := JoypadType.UNINDENTIFIED
	
	if joypad_name.find("PS") != -1 or \
		joypad_name.find("Sony") != -1 or \
		joypad_name == "Nacon Revolution Unlimited Pro Controller":
		type = JoypadType.PLAYSTATION
	elif joypad_name.find("Nintendo") != -1 or \
		joypad_name.find("Switch") != -1 or \
		joypad_name.find("Joy-Con") != -1:
		type = JoypadType.NINTENDO
	elif joypad_name.find("Xbox") != -1 or \
		joypad_name.find("XBOX") != -1 or \
		joypad_name.find("X360") != -1 or \
		joypad_name.find("XInput") != -1:
		type = JoypadType.XBOX
	
	return type

static func get_joypad_id(controller_number: int) -> int:
	var joypads := get_real_joypads()
	if controller_number <= 0 or controller_number > joypads.size():
		return -1
	
	return joypads[controller_number - 1]

static func is_in_touch_mode() -> bool:
	return get_real_joypads().size() == 0 and (OS.get_name() == "Android" or OS.get_name() == "iOS")
