class_name MathUtils

static func angle_difference(angle1: float, angle2: float) -> float:
	var diff := fposmod(angle2 - angle1, TAU)
	return fposmod(2.0 * diff, TAU) - diff

static func positive_angle(angle: float) -> float:
	return fposmod(angle, TAU)

static func is_between_angles(angle: float, first_angle: float, second_angle: float) -> bool:
	var standardized_angle := positive_angle(angle)
	var standardized_first_angle := positive_angle(first_angle)
	var standardized_second_angle := positive_angle(second_angle)
	
	if standardized_first_angle <= standardized_second_angle:
		return standardized_angle >= standardized_first_angle and standardized_angle <= standardized_second_angle
	else:
		return standardized_angle >= standardized_first_angle or standardized_angle <= standardized_second_angle

static func closest_angle(angle: float, reference_angle: float) -> float:
	var other_notation: float
	if angle < 0:
		other_notation = TAU + angle
	else:
		other_notation = -TAU + angle
	
	var initial_distance := absf(angle - reference_angle)
	var other_distance := absf(other_notation - reference_angle)
	
	if initial_distance == other_distance:
		if angle < 0 and reference_angle < 0:
			return angle
		else:
			return other_notation
	elif initial_distance < other_distance:
		return angle
	else:
		return other_notation
