class_name GrenadeProjectile
extends Projectile

var use_direction_as_destination := false

var _init_pos: Vector2
var _dest_pos: Vector2
var _initial_angle: float
var _begin_angle: float
var _end_angle: float
var _curve_change_threshold: float
var _has_reached_final_position := false

func _get_real_max_distance() -> float:
	if use_direction_as_destination:
		return _init_pos.distance_to(_dest_pos)
	else:
		return max_distance

func _ready():
	Sounds.play_basic_projectile($ShootSound, weapon.get_volume_db_adjustment())
	
	_init_pos = global_position
	
	if use_direction_as_destination:
		_dest_pos = direction
		_initial_angle = _init_pos.direction_to(_dest_pos).angle()
	else:
		_dest_pos = _init_pos + (direction * _get_real_max_distance())
		_initial_angle = direction.angle()
	
	var middle_point := (_init_pos + _dest_pos) / 2
	var height := absf(_dest_pos.x - _init_pos.x) / 2
	var middle_point_in_air := middle_point + (Vector2.UP * height)
	
	_begin_angle = MathUtils.closest_angle(_init_pos.angle_to_point(middle_point_in_air), _initial_angle)
	
	if _initial_angle == _begin_angle:
		_end_angle = _initial_angle
	else:
		_end_angle = MathUtils.closest_angle(middle_point_in_air.angle_to_point(_dest_pos), _initial_angle)
	
	var first_distance := _init_pos.distance_to(middle_point_in_air)
	var seconde_distance := _dest_pos.distance_to(middle_point_in_air)
	_curve_change_threshold = first_distance / (first_distance + seconde_distance)
	
	direction = Vector2.from_angle(_begin_angle)
	if automatically_apply_rotation:
		rotation = direction.angle()

func _physics_process(delta: float):
	if not is_instance_valid(weapon):
		_destroy()
		return
	
	# We ignore all weapon motions with this emitter, we only use an hardcoded motion
	var movement := direction * 650 * delta
	if automatically_apply_rotation:
		rotation = movement.angle()
	
	if _has_reached_final_position:
		return # needed for landmine only
	
	var real_max_distance := _get_real_max_distance()
	var current_distance := _init_pos.distance_to(global_position)
	var progression := current_distance / real_max_distance # from 0 to 1
	
	if progression < _curve_change_threshold:
		movement /= lerp(1.0, 2.0, progression / _curve_change_threshold)
		direction = Vector2.from_angle(lerp(_begin_angle, _initial_angle, progression / _curve_change_threshold))
	else:
		movement /= lerp(2.0, 2 * _curve_change_threshold, (progression - _curve_change_threshold) / (1 - _curve_change_threshold))
		direction = Vector2.from_angle(lerp(_initial_angle, _end_angle, (progression - _curve_change_threshold) / (1 - _curve_change_threshold)))
	
	var collision := move_and_collide(movement)
	if collision:
		_has_reached_final_position = true
		_on_final_position_reached() # needed for landmine only
	else:
		current_distance = _init_pos.distance_to(global_position)
		if current_distance >= real_max_distance:
			_has_reached_final_position = true
			_on_final_position_reached()

func _on_final_position_reached():
	missed.emit(global_position)
	_miss()
