extends Projectile

const TIME_STEP := 1.0 / 60.0

var collision_normal := Vector2.ZERO

@onready var impact_particles := $ImpactParticles

var last_planned_movement: Vector2

# Moves the laser tracer along the path it would have taken over time as a
# regular projectile. Returns an array of positions of each point it reached
# along this line.
func trace_path() -> Array[Vector2]:
	impact_particles.emitting = true
	position = Vector2.ZERO # Don't remove, it is indeed important
	elapsed_time = 0.0
	
	var positions: Array[Vector2] = [global_position]
	var has_collided := false
	
	while not has_collided and positions.size() < 200 and positions.front().distance_to(global_position) < max_distance:
		last_planned_movement = _update_movement(TIME_STEP)
		var collision := move_and_collide(last_planned_movement)
		
		positions.append(global_position)
		
		if collision:
			collided.emit(collision.get_collider(), collision.get_position())
			
			collision_normal = collision.get_normal()
			_impact()
			
			has_collided = true
	
	if not has_collided:
		missed.emit(global_position)
		_miss()
	
	return positions

func _physics_process(_delta: float):
	pass

func stop():
	impact_particles.emitting = false

# Moves the impact particle effect to the laser and shows it.
func _impact():
	impact_particles.global_position = global_position
	impact_particles.rotation = collision_normal.angle()

# Turns off any impact particle effect currently running.
func _miss():
	impact_particles.global_position = global_position
	impact_particles.rotation = last_planned_movement.angle()
