class_name LightningJolt
extends Line2D

@export_range(0.5, 3.0) var spread_angle := PI/4.0
@export_range(1, 36) var segments := 12

@onready var sparks := $Sparks
@onready var ray_cast: RayCast2D = $RayCast2D

signal collided(target: Node, hit_location: Vector2)
signal missed(miss_location: Vector2)

func create(weapon: ModularWeapon, start: Vector2, end: Vector2):
	Sounds.play_electricity($Sound, weapon.get_volume_db_adjustment())
	
	ray_cast.global_position = start
	
	ray_cast.target_position = end - start
	ray_cast.force_raycast_update()
	
	if ray_cast.is_colliding():
		end = ray_cast.get_collision_point()
		collided.emit(ray_cast.get_collider(), end)
	else:
		missed.emit(end)
	
	var _points := []
	var _current := start
	var _segment_length := start.distance_to(end) / segments
	
	_points.append(start)
	
	for segment in range(segments):
		# Face the end point and extend towards it
		# Rotate a random amount to get next point
		var _rotation := randf_range(-spread_angle / 2, spread_angle / 2)
		var _new := _current + (_current.direction_to(end) * _segment_length).rotated(_rotation)
		_points.append(_new)
		_current = _new
	
	_points.append(end)
	points = _points
	
	sparks.global_position = end
	sparks.emitting = true
