class_name WaveEmitter
extends ProjectileEmitter

var _projectile := preload("res://src/weapon/ProjectileEmitters/Wave/WaveProjectile.tscn")
var fire_holed_circle := false

@onready var timer := $Timer

func _ready():
	damage_per_collision = 80

# Spawns and configures a basic projectile scene and connects to its signals.
func _do_fire(_delta: float, _direction: Vector2) -> bool:
	if not timer.is_stopped():
		return false
	
	var new_projectile := _projectile.instantiate() as WaveProjectile
	if fire_holed_circle:
		new_projectile.arc_mi_angle = PI / 1.15
	new_projectile.setup(weapon, global_position, _direction.rotated(global_rotation), _get_real_max_distance())
	EventBus.create_projectile.emit(new_projectile)
	
	new_projectile.collided.connect(_on_projectile_collided)
	new_projectile.missed.connect(_on_projectile_missed)
	
	timer.start(1.0 / _get_real_projectiles_per_second())
	
	return true

# Emits the damaged signal and triggers any impact event in the system.
func _on_projectile_collided(target: Node, hit_location: Vector2):
	# x2 damage when close, /2 when distant
	var wave_damage_factor := 0.5 + global_position.distance_to(target.global_position) / _get_real_max_distance()
	
	weapon.damaged.emit(target, damage_per_collision / wave_damage_factor)
	for impact in weapon.projectile_impacts:
		(impact as ProjectileImpact).trigger(hit_location, weapon, false, target)
