class_name WaveProjectile
extends Projectile

const _wave_thickness := 10.0
var arc_mi_angle := PI / 20
var _wave_color: Color
var _secondary_wave_color := Color.WHITE

var _radius := 0.0
var _in_circle_collisions: Array[Node2D] = []
var _in_arc_collisions: Array[Node2D] = []


func _ready():
	Sounds.play_wave_projectile($ShootSound, weapon.get_volume_db_adjustment())
	
	rotation = direction.angle()
	
	if not weapon.player:
		z_index = 10
		$Area2D.call_deferred("set_collision_mask_value", 4, false)
		$Area2D.call_deferred("set_collision_mask_value", 2, true)
	
	$Area2D/CollisionShape2D.shape.radius = 0
	
	if weapon.player:
		_wave_color = Color.CORNFLOWER_BLUE
	else:
		_wave_color = Color.RED

func _physics_process(_delta: float):
	pass

func _process(delta: float):
	if not is_instance_valid(weapon):
		_destroy()
		return
	
	var movement := _update_movement(delta)
	rotation = movement.angle()
	
	for body in _in_circle_collisions:
		if not is_instance_valid(body):
			_in_arc_collisions.erase(body)
			continue
		
		var distance_from_center := global_position.distance_to(body.global_position)
		var body_angle := global_position.angle_to_point(body.global_position)
		
		# Since we do not look at body collision shape, we artificially increase the collision angle
		var angle_adjustement := 0.2
		if body is Enemy and (body as Enemy)._nb_phases > 1: # If it's a boss, we increase the angle because it is larger
			angle_adjustement = 0.3
		angle_adjustement -= _radius / 2000 # we decrease that angle when going far
		
		if distance_from_center > _radius - _wave_thickness and MathUtils.is_between_angles(body_angle, rotation - arc_mi_angle - angle_adjustement, rotation + arc_mi_angle + angle_adjustement):
			if _in_arc_collisions.find(body) == -1:
				collided.emit(body, body.global_position)
				_in_arc_collisions.append(body)
		else:
			_in_arc_collisions.erase(body)
	
	_radius = 300.0 * elapsed_time
	
	if _radius > max_distance:
		queue_free()
	else:
		$Area2D/CollisionShape2D.shape.radius = _radius
		_draw_arcs()

func _draw_arcs():
	if _radius > 20:
		var thickness := _wave_thickness * (1 - (_radius / max_distance))
		
		DrawUtils.draw_arc($FirstLine2D, _radius - _wave_thickness, _wave_color, thickness, -arc_mi_angle, arc_mi_angle)
		DrawUtils.draw_arc($SecondLine2D, _radius, _secondary_wave_color, thickness, -arc_mi_angle, arc_mi_angle)

func _on_area_2d_body_entered(body: Node2D):
	_in_circle_collisions.append(body)

func _on_area_2d_body_exited(body: Node2D):
	_in_circle_collisions.erase(body)
	_in_arc_collisions.erase(body)

# The default animation (scale) does not work for wave, we prefer a fadeout
func _destroy():
	_stop_physics()
	
	var tween = get_tree().create_tween().bind_node(self)
	tween.tween_property(self, "modulate", Color.TRANSPARENT, 0.1)
	tween.tween_callback(self.queue_free)
