class_name Shockwave
extends Area2D

var weapon: ModularWeapon
var entity_to_ignore: Node2D
var shockwave_max_radius: float
var shockwave_damage: int
var shockwave_width := 15.0

var _elapsed_time := 0.0
var _radius := 0.0

var _in_circle_collisions: Array[Node2D] = []
var _in_ring_collisions: Array[Node2D] = []

var _shockwave_color: Color
var _shockwave_secondary_color := Color.WHITE

func _ready():
	if not weapon.player:
		z_index = 10
		call_deferred("set_collision_mask_value", 4, false)
		call_deferred("set_collision_mask_value", 2, true)
	
	if weapon.can_destroy_objects:
		call_deferred("set_collision_mask_value", 6, true)
	
	$CollisionShape2D.shape.radius = 0
	
	if weapon.player:
		_shockwave_color = Color.ORANGE
		_shockwave_color.a = 0.5
		_shockwave_secondary_color.a = 0.5
	else:
		_shockwave_color = Color.RED
		_shockwave_color.a = 0.7
		_shockwave_secondary_color.a = 0.7

func _process(delta: float):
	if not is_instance_valid(weapon):
		queue_free()
		return
	
	for body in _in_circle_collisions:
		if is_instance_valid(body) and global_position.distance_to(body.global_position) > _radius - shockwave_width:
			if body != entity_to_ignore and _in_ring_collisions.find(body) == -1:
				weapon.damaged.emit(body, shockwave_damage)
				_in_ring_collisions.append(body)
		else:
			_in_ring_collisions.erase(body)
	
	_elapsed_time += delta
	_radius = 200.0 * _elapsed_time
	
	if _radius > shockwave_max_radius:
		queue_free()
	else:
		$CollisionShape2D.shape.radius = _radius
		_draw_circles()

func _draw_circles():
	if _radius > 2:
		var width := shockwave_width
		if _radius > shockwave_max_radius - (shockwave_width * 3):
			width = shockwave_width * ((shockwave_max_radius - _radius) / (shockwave_width * 3))
		
		DrawUtils.draw_circle($FirstLine2D, _radius - (width / 2.0), _shockwave_color, width / 4.0)
		DrawUtils.draw_circle($SecondLine2D, _radius, _shockwave_secondary_color, width / 5.0)
		DrawUtils.draw_circle($ThirdLine2D, _radius + (width / 2.0), _shockwave_color, width / 4.0)

func _on_body_entered(body: Node2D):
	_in_circle_collisions.append(body)

func _on_body_exited(body: Node2D):
	_in_circle_collisions.erase(body)
	_in_ring_collisions.erase(body)
